// Defined to disable deprecated functions warning in Visual C++ 2005
#define _CRT_SECURE_NO_DEPRECATE 1

#include <windows.h>
#include <stdio.h>

#pragma pack(1)       // set byte packing
typedef struct {
	unsigned __int32 bLength;
	char szSerNum[9];
	char szProdName[9];
	unsigned __int64 MinFreq;
	unsigned __int64 MaxFreq;
	struct {
		unsigned __int32 ExtRef:1;
		unsigned __int32 FMWEnabled:1;
		unsigned __int32 Reserved:30;
	} Features;
} RADIO_INFO2;
#pragma pack()       // set back the default packing

// G305 API function type declarations
typedef int (__stdcall *FNCOpenRadioDevice)(int iDeviceNum);
typedef BOOL (__stdcall *FNCCloseRadioDevice)(int hRadio);
typedef BOOL (__stdcall *FNCGetInfo)(int hRadio,RADIO_INFO2 *info);
typedef BOOL (__stdcall *FNCSetAtten)(int hRadio, BOOL fAtten);
typedef BOOL (__stdcall *FNCSetPower)(int hRadio, BOOL fPower);
typedef BOOL (__stdcall *FNCSetAGC)(int hRadio, int iAGC);
typedef BOOL (__stdcall *FNCGetFMW)(int hRadio);
typedef BOOL (__stdcall *FNCInvertSpectrum)(int hRadio,void *Samples,size_t SamplesNum,float SamplingRate,size_t BitsPerSample,BOOL Force);
typedef BOOL (__stdcall *FNCCodecStart)(int hRadio,void (__stdcall *CallbackFunc)(void *),void *CallbackTarget);
typedef BOOL (__stdcall *FNCCodecStop)(int hRadio);
typedef size_t (__stdcall *FNCCodecRead)(int hRadio,void *Buf,size_t Size);

typedef struct {
	int hRadio;
	FILE *IFFile;
	FNCCodecRead CodecRead;
	FNCGetFMW GetFMW;
	FNCInvertSpectrum InvertSpectrum;
} CallbackContext;

void __stdcall CodecCallback(void *Target)
{
	size_t i;
	char Buf[4096];
	CallbackContext *Ctx=(CallbackContext *)Target;
	static DWORD OldByte=0xFFFFFFFF;

	while ((i=Ctx->CodecRead(Ctx->hRadio,(OldByte<0x00000100)?(Buf+1):Buf,(OldByte<0x00000100)?(sizeof(Buf)-1):sizeof(Buf)))!=0) {
		// if a previosuly stored byte is available it must be recovered
		if (OldByte<0x00000100) {
			Buf[0]=(char)OldByte;
			OldByte=0xFFFFFFFF;
			i++;
		}
		// if odd number of bytes, the last byte must be stored for later
		if (i%2) {
			i--;
			OldByte=Buf[i];
		}
		// if not in FMW mode it must apply spectrum correction
		if (!Ctx->GetFMW(Ctx->hRadio))
			Ctx->InvertSpectrum(Ctx->hRadio,Buf,i/sizeof(WORD),64000,16,FALSE);
		fwrite(Buf,i,1,Ctx->IFFile);
	}
}

int main(int argc, char* argv[])
{
	// load the G3 API library
	HMODULE dll=LoadLibrary("wrg305api.dll");
	if (!dll) {
		puts("WRG305API.DLL not found !"); 
		return 0;
	}

	// link G3 API functions
	FNCOpenRadioDevice OpenRadioDevice=(FNCOpenRadioDevice)GetProcAddress(dll,"OpenRadioDevice");
	FNCCloseRadioDevice CloseRadioDevice=(FNCCloseRadioDevice)GetProcAddress(dll,"CloseRadioDevice");
	FNCGetInfo GetInfo=(FNCGetInfo)GetProcAddress(dll,"GetInfo");
	FNCSetAtten SetAtten=(FNCSetAtten)GetProcAddress(dll,"SetAtten");
	FNCSetPower SetPower=(FNCSetPower)GetProcAddress(dll,"SetPower");
	FNCSetAGC SetAGC=(FNCSetAGC)GetProcAddress(dll,"SetAGC");
	FNCGetFMW GetFMW=(FNCGetFMW)GetProcAddress(dll,"GetFMW");
	FNCInvertSpectrum InvertSpectrum=(FNCInvertSpectrum)GetProcAddress(dll,"InvertSpectrum");
	FNCCodecStart CodecStart=(FNCCodecStart)GetProcAddress(dll,"CodecStart");
	FNCCodecStop CodecStop=(FNCCodecStop)GetProcAddress(dll,"CodecStop");
	FNCCodecRead CodecRead=(FNCCodecRead)GetProcAddress(dll,"CodecRead");

	if (!GetInfo || !InvertSpectrum) {
		puts("Not all needed API function not found. Please update the WR-G305 software!");
		FreeLibrary(dll);
		return 0;
	}

	// open the first available radio
	int hRadio=OpenRadioDevice(0);
	if (!hRadio) {
		puts("No WR-G305 device could be opened !");
		FreeLibrary(dll); 
		return 0;
	}

	// obtain the radio info to be able to detect USB connected receivers
	RADIO_INFO2 radio_info;
	radio_info.bLength=sizeof(radio_info);
	GetInfo(hRadio,&radio_info);

	// check for USB receiver
	if (memcmp(radio_info.szProdName,"G3USBv",6))
		puts("Detected receiver is not connected through USB and codec functions are not available");
	else {
		// set Power, AGC and Attenuator
		if (SetPower(hRadio,TRUE)) puts("The device is turned ON"); 
		else puts("The device failed to turn ON");
		if (SetAtten(hRadio,FALSE)) puts("The attenuator is OFF"); 
		else puts("The attenuator failed to turn OFF");
		if (SetAGC(hRadio,3)) puts("The AGC is FAST"); 
		else puts("The AGC failed to switch");

		// open the IF samples destination file
		FILE *f=fopen("IF.pcm","wb");

		// check for valid output file
		if (!f) puts("Error while opening the output file");
		else {
			// start the IF stream from the codec
			CallbackContext Ctx;

			Ctx.hRadio=hRadio;
			Ctx.IFFile=f;
			Ctx.CodecRead=CodecRead;
			Ctx.GetFMW=GetFMW;
			Ctx.InvertSpectrum=InvertSpectrum;
			if (!CodecStart(hRadio,CodecCallback,&Ctx)) puts("Codec streaming couldn't be started!");
			else {
				puts("Codec streaming properly started!");
				while (ftell(f)<1280000) {}
				CodecStop(hRadio);
				fclose(f);
			}
		}
	}

	// close the device handle
	if (CloseRadioDevice(hRadio)) puts("The device is closed properly"); 
	else puts("The device failed to close");

	// free the G3 API library
	FreeLibrary(dll);

	return 0;
}
