// Defined to disable deprecated functions warning in Visual C++ 2005
#define _CRT_SECURE_NO_DEPRECATE 1

#include <windows.h>
#include <stdio.h>

#pragma pack(1)       // set byte packing
typedef struct {
	unsigned __int32 bLength;
	char szSerNum[9];
	char szProdName[9];
	unsigned __int64 MinFreq;
	unsigned __int64 MaxFreq;
	struct {
		unsigned __int32 ExtRef:1;
		unsigned __int32 FMWEnabled:1;
		unsigned __int32 Reserved:30;
	} Features;
} RADIO_INFO2;
#pragma pack()       // set back the default packing

// G305 API function type declarations
typedef int (__stdcall *FNCOpenRadioDevice)(int iDeviceNum);
typedef BOOL (__stdcall *FNCCloseRadioDevice)(int hRadio);
typedef BOOL (__stdcall *FNCGetInfo)(int hRadio,RADIO_INFO2 *info);
typedef BOOL (__stdcall *FNCSetAtten)(int hRadio, BOOL fAtten);
typedef BOOL (__stdcall *FNCSetPower)(int hRadio, BOOL fPower);
typedef BOOL (__stdcall *FNCSetAGC)(int hRadio, int iAGC);
typedef int (__stdcall *FNCSetFreqPipelined)(int hRadio,DWORD dwFreq,BOOL IsFirst,LONG Settling);
typedef int (__stdcall *FNCGetSSPipelined)(int hRadio,int *SS,BYTE *RawSS,int *RSSI,int *AGC);
typedef BOOL (__stdcall *FNCCodecStart)(int hRadio,void (__stdcall *CallbackFunc)(void *),void *CallbackTarget);
typedef BOOL (__stdcall *FNCCodecStop)(int hRadio);

// Formula for the frequencies to be scanned
#define ScanFreqsNum	1000
#define ScanFreq(i)		((DWORD)(88000000+(i)*(20000000/ScanFreqsNum)))

int main(int argc, char* argv[])
{
	// load the G3 API library
	HMODULE dll=LoadLibrary("wrg305api.dll");
	if (!dll) {
		puts("WRG305API.DLL not found !"); 
		return 0;
	}

	// link G3 API functions
	FNCOpenRadioDevice OpenRadioDevice=(FNCOpenRadioDevice)GetProcAddress(dll,"OpenRadioDevice");
	FNCCloseRadioDevice CloseRadioDevice=(FNCCloseRadioDevice)GetProcAddress(dll,"CloseRadioDevice");
	FNCGetInfo GetInfo=(FNCGetInfo)GetProcAddress(dll,"GetInfo");
	FNCSetAtten SetAtten=(FNCSetAtten)GetProcAddress(dll,"SetAtten");
	FNCSetPower SetPower=(FNCSetPower)GetProcAddress(dll,"SetPower");
	FNCSetAGC SetAGC=(FNCSetAGC)GetProcAddress(dll,"SetAGC");
	FNCSetFreqPipelined SetFreqPipelined=(FNCSetFreqPipelined)GetProcAddress(dll,"SetFreqPipelined");
	FNCGetSSPipelined GetSSPipelined=(FNCGetSSPipelined)GetProcAddress(dll,"GetSSPipelined");
	FNCCodecStart CodecStart=(FNCCodecStart)GetProcAddress(dll,"CodecStart");
	FNCCodecStop CodecStop=(FNCCodecStop)GetProcAddress(dll,"CodecStop");

	if (!GetInfo || !SetFreqPipelined || !GetSSPipelined) {
		puts("Not all needed API function not found. Please update the WR-G305 software!");
		FreeLibrary(dll);
		return 0;
	}

	// open the first available radio
	int hRadio=OpenRadioDevice(0);
	if (!hRadio) {
		puts("No WR-G305 device could be opened !");
		FreeLibrary(dll); 
		return 0;
	}

	// obtain the radio info to be able to detect USB connected receivers
	RADIO_INFO2 radio_info;
	radio_info.bLength=sizeof(radio_info);
	GetInfo(hRadio,&radio_info);

	// check for USB receiver
	BOOL USB_recv=FALSE;
	if (!memcmp(radio_info.szProdName,"G3USBv",6)) {
		puts("USB receiver detected");
		USB_recv=TRUE;
	}

	// set Power,AGC and Attenuator
	if (SetPower(hRadio,TRUE)) puts("The device is turned ON");
	else puts("The device failed to turn ON");
	if (SetAtten(hRadio,FALSE)) puts("The attenuator is OFF");
	else puts("The attenuator failed to turn OFF");
	if (SetAGC(hRadio,3)) puts("The AGC is FAST");
	else puts("The AGC failed to switch");

	// for USB receivers codec streaming must be started
	if (USB_recv) CodecStart(hRadio,NULL,NULL);

	// let's start the pipelined scanning by passing the first frequency
	if (SetFreqPipelined(hRadio,ScanFreq(0),TRUE,20)) puts("Error while pipelining the first frequency..");
	else {
		size_t i;

		puts("First frequency successfully pipelined..");
		// now it does all steps that can be done in a continous loop
		for (i=1;i<=ScanFreqsNum;i++) {
			// pipelines the next frequency
			if (SetFreqPipelined(hRadio,ScanFreq(i),FALSE,3)) printf("Error while pipelining %u..\n",ScanFreq(i));
			else {
				int SS,RSSI,AGC,Res;
				BYTE RawSS;

				Res=GetSSPipelined(hRadio,&SS,&RawSS,&RSSI,&AGC);
				if (Res) printf("Error while reading SS for %u..\n",ScanFreq(i-1));
				else printf("%u - SS=%d   RawSS=%u   AGC=%d   RSSI=%d\n",ScanFreq(i-1),SS,RawSS,AGC,RSSI);
			}
		}
		// the first step requires a dummy frequency pipelining
		if (SetFreqPipelined(hRadio,ScanFreq(ScanFreqsNum),FALSE,3)) printf("Error while re-pipelining %u..\n",ScanFreq(ScanFreqsNum));
		else {
			int SS,RSSI,AGC,Res;
			BYTE RawSS;

			Res=GetSSPipelined(hRadio,&SS,&RawSS,&RSSI,&AGC);
			if (Res) printf("Error while reading SS for %u..\n",ScanFreq(ScanFreqsNum));
			else printf("%u - SS=%d   RawSS=%u   AGC=%d   RSSI=%d\n",ScanFreq(ScanFreqsNum),SS,RawSS,AGC,RSSI);
		}
	}

	// for USB receivers codec streaming must be stopped
	if (USB_recv) CodecStop(hRadio);

	// close the device handle
	if (CloseRadioDevice(hRadio)) puts("The device is closed properly");
	else puts("The device failed to close");

	// free the G3 API library
	FreeLibrary(dll);

	return 0;
}
